gcc/
        * function.h (struct rtl_data): Convert field
        "x_nonlocal_goto_handler_labels" from rtx_expr_list * to
        rtx_insn_list *.
        * rtl.h (remove_node_from_insn_list): New prototype.

        * builtins.c (expand_builtin): When prepending to
        nonlocal_goto_handler_labels, use gen_rtx_INSN_LIST rather than
        gen_rtx_EXPR_LIST.
        * cfgbuild.c (make_edges): Convert local "x" from rtx_expr_list *
        to rtx_insn_list *, and use its "insn" method rather than
        "element" method.
        * cfgrtl.c (delete_insn): Use new function
        remove_node_from_insn_list rather than
        remove_node_from_expr_list.
        (cfg_layout_initialize): Convert local "x" from rtx_expr_list *
        to rtx_insn_list *, and use its "insn" method rather than
        "element" method.
        * dwarf2cfi.c (create_trace_edges): Likewise for local "lab".
        * reload1.c (set_initial_label_offsets): Likewise for local "x".
        * rtlanal.c (remove_node_from_insn_list): New function, adapted
        from remove_node_from_expr_list.
        * stmt.c (expand_label): When prepending to
        nonlocal_goto_handler_labels, use gen_rtx_INSN_LIST rather than
        gen_rtx_EXPR_LIST.
---
 gcc/builtins.c  |  2 +-
 gcc/cfgbuild.c  |  4 ++--
 gcc/cfgrtl.c    |  6 +++---
 gcc/dwarf2cfi.c |  4 ++--
 gcc/function.h  |  4 ++--
 gcc/reload1.c   |  6 +++---
 gcc/rtl.h       |  1 +
 gcc/rtlanal.c   | 29 +++++++++++++++++++++++++++++
 gcc/stmt.c      |  2 +-
 9 files changed, 44 insertions(+), 14 deletions(-)

diff --git a/gcc/builtins.c b/gcc/builtins.c
index e5a9b4d..5b1068e 100644
--- a/gcc/builtins.c
+++ b/gcc/builtins.c
@@ -6186,7 +6186,7 @@ expand_builtin (tree exp, rtx target, rtx subtarget, enum 
machine_mode mode,
          /* This is copied from the handling of non-local gotos.  */
          expand_builtin_setjmp_setup (buf_addr, label_r);
          nonlocal_goto_handler_labels
-           = gen_rtx_EXPR_LIST (VOIDmode, label_r,
+           = gen_rtx_INSN_LIST (VOIDmode, label_r,
                                 nonlocal_goto_handler_labels);
          /* ??? Do not let expand_label treat us as such since we would
             not want to be both on the list of non-local labels and on
diff --git a/gcc/cfgbuild.c b/gcc/cfgbuild.c
index 082f070..d7fa97a 100644
--- a/gcc/cfgbuild.c
+++ b/gcc/cfgbuild.c
@@ -337,10 +337,10 @@ make_edges (basic_block min, basic_block max, int 
update_p)
                     taken, then only calls to those functions or to other
                     nested functions that use them could possibly do
                     nonlocal gotos.  */
-                 for (rtx_expr_list *x = nonlocal_goto_handler_labels;
+                 for (rtx_insn_list *x = nonlocal_goto_handler_labels;
                       x;
                       x = x->next ())
-                   make_label_edge (edge_cache, bb, x->element (),
+                   make_label_edge (edge_cache, bb, x->insn (),
                                     EDGE_ABNORMAL | EDGE_ABNORMAL_CALL);
                }
 
diff --git a/gcc/cfgrtl.c b/gcc/cfgrtl.c
index 9707c1f..d9b41e0 100644
--- a/gcc/cfgrtl.c
+++ b/gcc/cfgrtl.c
@@ -159,7 +159,7 @@ delete_insn (rtx uncast_insn)
            }
        }
 
-      remove_node_from_expr_list (insn, &nonlocal_goto_handler_labels);
+      remove_node_from_insn_list (insn, &nonlocal_goto_handler_labels);
     }
 
   if (really_delete)
@@ -4216,7 +4216,7 @@ cfg_layout_duplicate_bb (basic_block bb)
 void
 cfg_layout_initialize (unsigned int flags)
 {
-  rtx_expr_list *x;
+  rtx_insn_list *x;
   basic_block bb;
 
   /* Once bb partitioning is complete, cfg layout mode should not be
@@ -4237,7 +4237,7 @@ cfg_layout_initialize (unsigned int flags)
   /* Make sure that the targets of non local gotos are marked.  */
   for (x = nonlocal_goto_handler_labels; x; x = x->next ())
     {
-      bb = BLOCK_FOR_INSN (x->element ());
+      bb = BLOCK_FOR_INSN (x->insn ());
       bb->flags |= BB_NON_LOCAL_GOTO_TARGET;
     }
 
diff --git a/gcc/dwarf2cfi.c b/gcc/dwarf2cfi.c
index 2452824..7c495e4 100644
--- a/gcc/dwarf2cfi.c
+++ b/gcc/dwarf2cfi.c
@@ -2338,10 +2338,10 @@ create_trace_edges (rtx insn)
 
       /* Process non-local goto edges.  */
       if (can_nonlocal_goto (insn))
-       for (rtx_expr_list *lab = nonlocal_goto_handler_labels;
+       for (rtx_insn_list *lab = nonlocal_goto_handler_labels;
             lab;
             lab = lab->next ())
-         maybe_record_trace_start_abnormal (lab->element (), insn);
+         maybe_record_trace_start_abnormal (lab->insn (), insn);
     }
   else if (rtx_sequence *seq = dyn_cast <rtx_sequence *> (PATTERN (insn)))
     {
diff --git a/gcc/reload1.c b/gcc/reload1.c
index 9db479e..419940e 100644
--- a/gcc/reload1.c
+++ b/gcc/reload1.c
@@ -3915,9 +3915,9 @@ set_initial_label_offsets (void)
     if (x->element ())
       set_label_offsets (x->element (), NULL, 1);
 
-  for (rtx_expr_list *x = nonlocal_goto_handler_labels; x; x = x->next ())
-    if (x->element ())
-      set_label_offsets (x->element (), NULL, 1);
+  for (rtx_insn_list *x = nonlocal_goto_handler_labels; x; x = x->next ())
+    if (x->insn ())
+      set_label_offsets (x->insn (), NULL, 1);
 
   for_each_eh_label (set_initial_eh_label_offset);
 }
diff --git a/gcc/rtl.h b/gcc/rtl.h
index 0732309..f34a3bd 100644
--- a/gcc/rtl.h
+++ b/gcc/rtl.h
@@ -2839,6 +2839,7 @@ extern rtx regno_use_in (unsigned int, rtx);
 extern int auto_inc_p (const_rtx);
 extern int in_expr_list_p (const_rtx, const_rtx);
 extern void remove_node_from_expr_list (const_rtx, rtx_expr_list **);
+extern void remove_node_from_insn_list (const rtx_insn *, rtx_insn_list **);
 extern int loc_mentioned_in_p (rtx *, const_rtx);
 extern rtx_insn *find_first_parameter_load (rtx, rtx);
 extern bool keep_with_call_p (const_rtx);
diff --git a/gcc/rtlanal.c b/gcc/rtlanal.c
index 297ca41..8383cdf 100644
--- a/gcc/rtlanal.c
+++ b/gcc/rtlanal.c
@@ -2156,6 +2156,35 @@ remove_node_from_expr_list (const_rtx node, 
rtx_expr_list **listp)
       temp = temp->next ();
     }
 }
+
+/* Search LISTP (an INSN_LIST) for an entry whose first operand is NODE and
+   remove that entry from the list if it is found.
+
+   A simple equality test is used to determine if NODE matches.  */
+
+void
+remove_node_from_insn_list (const rtx_insn *node, rtx_insn_list **listp)
+{
+  rtx_insn_list *temp = *listp;
+  rtx prev = NULL;
+
+  while (temp)
+    {
+      if (node == temp->insn ())
+       {
+         /* Splice the node out of the list.  */
+         if (prev)
+           XEXP (prev, 1) = temp->next ();
+         else
+           *listp = temp->next ();
+
+         return;
+       }
+
+      prev = temp;
+      temp = temp->next ();
+    }
+}
 
 /* Nonzero if X contains any volatile instructions.  These are instructions
    which may cause unpredictable machine state instructions, and thus no
diff --git a/gcc/stmt.c b/gcc/stmt.c
index beafaa0..1cbd63d 100644
--- a/gcc/stmt.c
+++ b/gcc/stmt.c
@@ -187,7 +187,7 @@ expand_label (tree label)
     {
       expand_builtin_setjmp_receiver (NULL);
       nonlocal_goto_handler_labels
-       = gen_rtx_EXPR_LIST (VOIDmode, label_r,
+       = gen_rtx_INSN_LIST (VOIDmode, label_r,
                             nonlocal_goto_handler_labels);
     }
 
diff --git a/gcc/function.h b/gcc/function.h
index c2e0366..3921d21 100644
--- a/gcc/function.h
+++ b/gcc/function.h
@@ -264,9 +264,9 @@ struct GTY(()) rtl_data {
      Used for detecting stack clobbers.  */
   tree stack_protect_guard;
 
   /* List (chain of EXPR_LIST) of labels heading the current handlers for
      nonlocal gotos.  */
-  rtx_expr_list *x_nonlocal_goto_handler_labels;
+  rtx_insn_list *x_nonlocal_goto_handler_labels;
 
   /* Label that will go on function epilogue.
      Jumping to this label serves as a "return" instruction
@@ -264,9 +264,9 @@ struct GTY(()) rtl_data {
   tree stack_protect_guard;
 
-  /* List (chain of EXPR_LIST) of labels heading the current handlers for
+  /* List (chain of INSN_LIST) of labels heading the current handlers for
      nonlocal gotos.  */
-- 
1.8.5.3

Reply via email to